# noinspection PyUnresolvedReferences
from myorm.factory import MySQLSessionFactoryBuild as Builder
# noinspection PyUnresolvedReferences
from superorm.orm import parse_config_from_file, parse_config_from_string



from jinja2 import Environment

from myorm.info import DataBaseInfo

# noinspection SpellCheckingInspection
_mapper_xml = """
<xml>
    <sql>
        <key>GetList</key>
        <value>
            SELECT
                {% for column in table.columns %}`{{ column.Name }}`{% if not loop.last %}, {% endif %}{% endfor %}
            FROM
                `{{table.Name}}`
            WHERE
    {% for column in table.columns %}
        {% if column.Name != key %}
            {% if column.Type|clear_type in int_types %}
                {{ "{"|echo }}% if {{ column.Name }} and {{ column.Name }} != 0 %{{ "}"|echo }}
                    `{{ column.Name }}` = #{ {{ column.Name }} } AND
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
            {% if column.Type|clear_type in float_types %}
                {{ "{"|echo }}% if {{ column.Name }} and {{ column.Name }} != 0 %{{ "}"|echo }}
                    `{{ column.Name }}` = #{ {{ column.Name }} } AND
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
            {% if column.Type|clear_type in date_types %}
                {% if column.Type|clear_type == "date" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y-%m-%d") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "time" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%H:%M:%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "year" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "timestamp" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y%m%d%H%M%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "datetime" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y-%m-%d %H:%M:%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
            {% endif %}
            {% if column.Type|clear_type in string_types %}
                {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }} 
                    `{{ column.Name }}` LIKE #{ {{ column.Name }} } AND 
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
        {% endif %}
    {% endfor %}
            1 = 1
            {{ "{"|echo }}% if (mysql_mapper_order_by | default("")) != "" %{{ "}"|echo }} 
                ORDER BY {{ "{"|echo }}{ mysql_mapper_order_by }{{ "}"|echo }} 
            {{ "{"|echo }}% endif %{{ "}"|echo }} 
            {{ "{"|echo }}% if (mysql_mapper_limit_start | default(-1)) != -1 %{{ "}"|echo }} 
                LIMIT #{ mysql_mapper_limit_start }, #{ mysql_mapper_limit_length | default(10) } 
            {{ "{"|echo }}% endif %{{ "}"|echo }}
        </value>
    </sql>
    <sql>
        <key>GetCount</key>
        <value>
            SELECT
                COUNT(1)
            FROM
                `{{table.Name}}`
            WHERE
    {% for column in table.columns %}
        {% if column.Name != key %}
            {% if column.Type|clear_type in int_types %}
                {{ "{"|echo }}% if {{ column.Name }} and {{ column.Name }} != 0 %{{ "}"|echo }}
                    `{{ column.Name }}` = #{ {{ column.Name }} } AND
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
            {% if column.Type|clear_type in float_types %}
                {{ "{"|echo }}% if {{ column.Name }} and {{ column.Name }} != 0 %{{ "}"|echo }}
                    `{{ column.Name }}` = #{ {{ column.Name }} } AND
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
            {% if column.Type|clear_type in date_types %}
                {% if column.Type|clear_type == "date" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y-%m-%d") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "time" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%H:%M:%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "year" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "timestamp" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y%m%d%H%M%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "datetime" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y-%m-%d %H:%M:%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
            {% endif %}
            {% if column.Type|clear_type in string_types %}
                {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }} 
                    `{{ column.Name }}` LIKE #{ {{ column.Name }} } AND 
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
        {% endif %}
    {% endfor %}
            1 = 1
        </value>
    </sql>
    <sql>
        <key>Exist</key>
        <value>
            SELECT
                COUNT(1)
            FROM
                `{{table.Name}}`
            WHERE
    {% for column in table.columns %}
        {% if column.Name != key %}
            {% if column.Type|clear_type in int_types %}
                {{ "{"|echo }}% if {{ column.Name }} and {{ column.Name }} != 0 %{{ "}"|echo }}
                    `{{ column.Name }}` = #{ {{ column.Name }} } AND
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
            {% if column.Type|clear_type in float_types %}
                {{ "{"|echo }}% if {{ column.Name }} and {{ column.Name }} != 0 %{{ "}"|echo }}
                    `{{ column.Name }}` = #{ {{ column.Name }} } AND
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
            {% if column.Type|clear_type in date_types %}
                {% if column.Type|clear_type == "date" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y-%m-%d") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "time" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%H:%M:%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "year" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "timestamp" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y%m%d%H%M%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "datetime" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y-%m-%d %H:%M:%S") } AND
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
            {% endif %}
            {% if column.Type|clear_type in string_types %}
                {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }} 
                    `{{ column.Name }}` LIKE #{ {{ column.Name }} } AND 
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
        {% endif %}
    {% endfor %}
            1 = 1
            LIMIT #{ mysql_mapper_limit_start }, #{ mysql_mapper_limit_length | default(10) }
        </value>
    </sql>
    <sql>
        <key>GetModel</key>
        <value>
            SELECT
                {% for column in table.columns %}`{{ column.Name }}`{% if not loop.last %}, {% endif %}{% endfor %}
            FROM
                `{{table.Name}}`
            WHERE
                `{{key}}` = #{ {{key}} }
        </value>
    </sql>
    <sql>
        <key>Insert</key>
        <value>
            INSERT INTO `{{table.Name}}`
            (
        {% for column in table.columns %}
            {% if column.Name != key or table.AutoIncrement == -1 %}
                `{{ column.Name }}`{% if not loop.last %}, {% endif %}
            {% endif %}
        {% endfor %}
            )
            VALUES
            (
    {% for column in table.columns %}
        {% if column.Name != key or table.AutoIncrement == -1 %}
            {% if column.Type|clear_type in int_types %}
                #{ {{column.Name}} }
            {% endif %}
            {% if column.Type|clear_type in float_types %}
                #{ {{column.Name}} }
            {% endif %}
            {% if column.Type|clear_type in date_types %}
                {% if column.Type|clear_type == "date" %}
                    #{ {{column.Name}}.strftime("%Y-%m-%d") }
                {% endif %}
                {% if column.Type|clear_type == "time" %}
                    #{ {{column.Name}}.strftime("%H:%M:%S") }
                {% endif %}
                {% if column.Type|clear_type == "year" %}
                    #{ {{column.Name}}.strftime("%Y") }
                {% endif %}
                {% if column.Type|clear_type == "timestamp" %}
                    #{ {{column.Name}}.strftime("%Y%m%d%H%M%S") }
                {% endif %}
                {% if column.Type|clear_type == "datetime" %}
                    #{ {{column.Name}}.strftime("%Y-%m-%d %H:%M:%S") }
                {% endif %}
            {% endif %}
            {% if column.Type|clear_type in string_types %}
                #{ {{column.Name}} }
            {% endif %}
            {% if not loop.last %}, {% endif %}
        {% endif %}
    {% endfor %}
            )
        </value>
    </sql>
    <sql>
        <key>Update</key>
        <value>
            UPDATE `{{table.Name}}` SET
    {% for column in table.columns %}
        {% if column.Name != key %}
            {% if column.Type|clear_type in int_types %}
                {{ "{"|echo }}% if {{ column.Name }} and {{ column.Name }} != 0 %{{ "}"|echo }}
                    `{{ column.Name }}` = #{ {{ column.Name }} },
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
            {% if column.Type|clear_type in float_types %}
                {{ "{"|echo }}% if {{ column.Name }} and {{ column.Name }} != 0 %{{ "}"|echo }}
                    `{{ column.Name }}` = #{ {{ column.Name }} },
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
            {% if column.Type|clear_type in date_types %}
                {% if column.Type|clear_type == "date" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y-%m-%d") },
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "time" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%H:%M:%S") },
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "year" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y") },
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "timestamp" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y%m%d%H%M%S") },
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
                {% if column.Type|clear_type == "datetime" %}
                    {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }}
                        `{{ column.Name }}` = #{ {{ column.Name }}.strftime("%Y-%m-%d %H:%M:%S") },
                    {{ "{"|echo }}% endif %{{ "}"|echo }}
                {% endif %}
            {% endif %}
            {% if column.Type|clear_type in string_types %}
                {{ "{"|echo }}% if {{ column.Name }} %{{ "}"|echo }} 
                    `{{ column.Name }}` = #{ {{ column.Name }} },
                {{ "{"|echo }}% endif %{{ "}"|echo }}
            {% endif %}
        {% endif %}
    {% endfor %}
            WHERE {{key}} = #{ {{key}} }
        </value>
    </sql>
    <sql>
        <key>Delete</key>
        <value>
            DELETE FROM `{{table.Name}}` WHERE `{{key}}` = #{ {{key}} }
        </value>
    </sql>
</xml>
"""


# noinspection SpellCheckingInspection
def get_mapper_xml(database_info: DataBaseInfo, table_name: str) -> str:
    """
    Building XML with database description information
    :param database_info: Database description information
    :param table_name: Table name
    :return: XML document
    """

    # Template environment
    env = Environment()

    # Custom filter
    def echo(value):
        """
        Print a character for secondary rendering
        :param value: Characters to be printed
        :return: Original output
        """
        return value

    def clear_type(value):
        """
        Database type cleanup
        :param value:  Database type
        :return: Cleaning results
        """
        return value.split("(")[0]

    # Loading filter
    env.filters['echo'] = echo
    env.filters['clear_type'] = clear_type

    # Loading template
    template = env.from_string(_mapper_xml)

    # Assemble render parameters
    data = {"data_base_name": database_info["Name"]}

    # Lookup table information
    table = None
    for item in database_info["tables"]:
        if item["Name"] == table_name:
            table = item
            break
    if table is None:
        return ""
    data["table"] = table

    # Find primary key information
    key = ""
    for item in table["indexs"]:
        if item["Name"] == "PRIMARY":
            key = item["ColumnName"]
            break
    if key is None:
        return ""
    data["key"] = key

    # Load supported data types
    data["int_types"] = ["bit", "tinyint", "smallint", "mediumint", "int", "integer", "bigint"]
    data["float_types"] = ["real", "double", "float", "decimal", "numeric"]
    data["date_types"] = ["date", "time", "year", "timestamp", "datetime"]
    data["string_types"] = ["char", "varchar", "text", "mediumtext", "longtext"]

    return template.render(data)



from myorm.mapper import get_mapper_xml
from myorm.service import Service
from superorm.dao import DAO
from superorm.logger import Logger
from superorm.factory import SQLSessionFactory, SQLSessionFactoryBuild
from superorm.manager import Manager
from superorm.mapper import parse4string

from myorm.engine import MySQLConnHandle, MySQLExecuteEngine, MySQLConnBuilder
from myorm.info import table_xml, column_xml, index_xml, key_xml, get_db_info, get_table_info


class MySQLSessionFactory(SQLSessionFactory):

    # Database conn info
    _conn_builder = None

    # Database description information
    database_info = None

    # Database table info
    table_info = None

    # Service dictionary
    _services = None

    def __init__(self, conn_builder: MySQLConnBuilder, conn_handle: MySQLConnHandle, execute_engine: MySQLExecuteEngine,
                 lazy_init: bool, max_conn_number: int, logger: Logger):
        """
        Init session pool
        :param conn_builder: MySQLConnBuilder
        :param conn_handle: MySQLConnHandle
        :param execute_engine: MySQLExecuteEngine
        :param lazy_init: lazy_init
        :param max_conn_number: max_conn_number
        :param logger: Logger
        """
        super().__init__(conn_builder, conn_handle, execute_engine, lazy_init, max_conn_number, logger)
        self._conn_builder = conn_builder
        self.load_dao("orm_table_info_dao", parse4string(table_xml))
        self.load_dao("orm_column_info_dao", parse4string(column_xml))
        self.load_dao("orm_index_info_dao", parse4string(index_xml))
        self.load_dao("orm_key_info_dao", parse4string(key_xml))
        self.database_info = {}
        self.table_info = {}
        self._services = {}

    def get_service(self, service_name: str) -> Service:
        """
        Get service
        :param service_name: service name
        :return: DAO
        """
        return self._services[service_name]

    def load_service(self):
        """
        load database service
        :return: self
        """
        _db_info = get_db_info(self._dao, self._conn_builder.database)
        return self._load_service(_db_info)

    def load_service_by_table_name(self, table_name: str):
        """
        used by load table,s service when close the simple service
        :param table_name: database table name
        :return: self
        """
        _db_info = get_table_info(self._dao, self._conn_builder.database, table_name)
        return self._load_service(_db_info)

    def _load_service(self, database_info: dict):
        """
        load service from database info
        :param database_info: database info
        :return: self
        """
        # for item table in database info
        for table in database_info["tables"]:
            # get mapper xml
            xml_string = get_mapper_xml(database_info, table["Name"])
            # parse to config
            config = parse4string(xml_string)
            # get manager
            manager = Manager(self._engine, config, self._logger)
            # get dao
            dao = DAO(self._session_manager, manager)

            # get the database info && set the table info
            self.table_info[table["Name"]] = table
            # get service
            self._services[table["Name"]] = Service(dao)
        # update the database info
        self.database_info["tables"] = list(self.table_info.values())
        return self


class MySQLSessionFactoryBuild(SQLSessionFactoryBuild):

    def __init__(self, host: str, user: str, password: str, database: str, port=3306, charset="utf8"):
        """
        Init
        :param host: host
        :param user: user
        :param password: password
        :param database: database
        :param port: data source port
        :param charset: charset
        """
        conn_builder = MySQLConnBuilder(host, user, password, database, port, charset)
        super().__init__(conn_builder, MySQLConnHandle(), MySQLExecuteEngine())

    def build(self) -> MySQLSessionFactory:
        return MySQLSessionFactory(
            self._conn_builder,
            self._conn_handle,
            self._execute_engine,
            self._lazy_init,
            self._max_conn_number,
            self._logger
        )



from typing import Dict

from superorm.dao import DAO
from superorm.engine import QueryResult, CountResult


class Service:
    """
    Basic service layer
    """
    _dao = None

    # Statement constants
    _get_list = "GetList"
    _get_count = "GetCount"
    _exist = "Exist"
    _get_model = "GetModel"
    _insert = "Insert"
    _update = "Update"
    _delete = "Delete"

    def __init__(self, dao: DAO):
        """
        Initialize service layer
        :param dao: Dao layer
        """
        self._dao = dao

    def get_list(self, parameter: Dict, **kwargs) -> QueryResult:
        """
        Get data list
        :param parameter: Search parameters
        :return: Data list
        """
        query_parameter = parameter.copy()
        # Compatible with previous writing
        if "Start" in query_parameter:
            query_parameter["mysql_mapper_limit_start"] = query_parameter["Start"]
        if "Length" in query_parameter:
            query_parameter["mysql_mapper_limit_length"] = query_parameter["Length"]
        # Processing parameters in kwargs
        if "order_by" in kwargs:
            query_parameter["mysql_mapper_order_by"] = kwargs["order_by"]
        if "start" in kwargs:
            query_parameter["mysql_mapper_limit_start"] = kwargs["start"]
        if "length" in kwargs:
            query_parameter["mysql_mapper_limit_length"] = kwargs["length"]
        return self._dao.query(self._get_list, query_parameter)

    def get_first(self, parameter: Dict, **kwargs) -> dict:
        """
        Get data list, then go back to the first record
        :param parameter: Search parameters
        :return: First Data
        """
        data_list = self.get_list(parameter, **kwargs)
        if len(data_list) == 0:
            return {}
        return data_list[0]

    def get_count(self, parameter: Dict) -> CountResult:
        """
        Quantity acquisition
        :param parameter: Search parameters
        :return: Number
        """
        return self._dao.count(self._get_count, parameter)

    def exist(self, parameter: Dict) -> bool:
        """
        Quantity acquisition, judge whether the quantity is greater than 0
        :param parameter: Search parameters
        :return: Number
        """
        query_parameter = parameter.copy()
        query_parameter["mysql_mapper_limit_start"] = 0
        query_parameter["mysql_mapper_limit_length"] = 1
        return self.get_count(parameter) > 1

    def get_model(self, parameter: Dict) -> Dict:
        """
        Get record entity
        :param parameter: Search parameters
        :return: Record entity
        """
        list_dict = self._dao.query(self._get_model, parameter)
        if len(list_dict) == 0:
            return {}
        return list_dict[0]

    def insert(self, parameter: Dict) -> int:
        """
        insert record
        :param parameter: insert data
        :return: Insert results
        """
        number, _ = self._dao.exec(self._insert, parameter)
        return number

    def update(self, parameter: Dict) -> int:
        """
        Update record
        :param parameter: Update data
        :return: Update results
        """
        _, number = self._dao.exec(self._update, parameter)
        return number

    def delete(self, parameter: Dict) -> int:
        """
        Delete data
        :param parameter: Delete data
        :return: Delete result
        """
        _, number = self._dao.exec(self._delete, parameter)
        return number



from typing import Dict

DAOMap = Dict
DataBaseInfo = Dict

# noinspection SpellCheckingInspection
table_xml = """
<xml>
    <mapper column="TABLE_NAME" parameter="Name"/>
    <mapper column="ENGINE" parameter="Engine"/>
    <mapper column="TABLE_COLLATION" parameter="Collation"/>
    <mapper column="TABLE_COMMENT" parameter="Comment"/>
    <mapper column="IFNULL(AUTO_INCREMENT, -1)" parameter="AutoIncrement"/>
    <sql>
        <key>GetList</key>
        <value>
            SELECT
                TABLE_NAME, ENGINE, TABLE_COLLATION, TABLE_COMMENT, IFNULL(AUTO_INCREMENT, -1)
            FROM
                information_schema.TABLES
            WHERE
                TABLE_SCHEMA = #{ data_base_name } AND TABLE_TYPE = 'BASE TABLE'
        </value>
    </sql>
    <sql>
        <key>GetModel</key>
        <value>
            SELECT
                TABLE_NAME, ENGINE, TABLE_COLLATION, TABLE_COMMENT, IFNULL(AUTO_INCREMENT, -1)
            FROM
                information_schema.TABLES
            WHERE
                TABLE_SCHEMA = #{ data_base_name } AND TABLE_TYPE = 'BASE TABLE'
                AND TABLE_NAME = #{ table_name }
        </value>
    </sql>
</xml>
"""

# noinspection SpellCheckingInspection
column_xml = """
<xml>
    <mapper column="ORDINAL_POSITION" parameter="Number"/>
    <mapper column="COLUMN_NAME" parameter="Name"/>
    <mapper column="COLUMN_TYPE" parameter="Type"/>
    <mapper column="IS_NULLABLE" parameter="NullAble"/>
    <mapper column="COLUMN_DEFAULT" parameter="Defaule"/>
    <mapper column="COLUMN_COMMENT" parameter="Comment"/>
    <sql>
        <key>GetList</key>
        <value>
            SELECT
                ORDINAL_POSITION,
                COLUMN_NAME,
                COLUMN_TYPE,
                IS_NULLABLE,
                IFNULL(COLUMN_DEFAULT, ''),
                COLUMN_COMMENT
            FROM
                information_schema.COLUMNS
            WHERE
                TABLE_SCHEMA = #{ data_base_name } AND TABLE_NAME = #{ table_name }
        </value>
    </sql>
</xml>
"""

index_xml = """
<xml>
    <mapper column="INDEX_NAME" parameter="Name"/>
    <mapper column="COLUMN_NAME" parameter="ColumnName"/>
    <mapper column="NON_UNIQUE" parameter="Unique"/>
    <mapper column="INDEX_TYPE" parameter="Type"/>
    <sql>
        <key>GetList</key>
        <value>
            SELECT
                INDEX_NAME,
                COLUMN_NAME,
                NON_UNIQUE,
                INDEX_TYPE
            FROM
                information_schema.STATISTICS
            WHERE
                TABLE_SCHEMA = #{ data_base_name } AND TABLE_NAME = #{ table_name }
        </value>
    </sql>
</xml>
"""

key_xml = """
<xml>
    <mapper column="COLUMN_NAME" parameter="ColumnName"/>
    <mapper column="REFERENCED_TABLE_NAME" parameter="RelyTable"/>
    <mapper column="REFERENCED_COLUMN_NAME" parameter="RelyColumnName"/>
    <sql>
        <key>GetList</key>
        <value>
            SELECT
                COLUMN_NAME, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME
            FROM
                information_schema.KEY_COLUMN_USAGE
            WHERE
                CONSTRAINT_NAME != 'PRIMARY' AND
                TABLE_SCHEMA = REFERENCED_TABLE_SCHEMA AND
                TABLE_SCHEMA = #{ data_base_name } AND TABLE_NAME = #{ table_name }
        </value>
    </sql>
</xml>
"""


# noinspection SpellCheckingInspection
def get_db_info(dao_map: DAOMap, database_name: str) -> DataBaseInfo:
    """
    Get database information
    :param dao_map: the database info dao
    :param database_name: Database name
    :return: database information
    """
    return _get_info(dao_map, "GetList", {"data_base_name": database_name})


# noinspection SpellCheckingInspection
def get_table_info(dao_map: DAOMap, database_name: str, table_name: str) -> DataBaseInfo:
    """
    Get database information
    :param dao_map: the database info dao
    :param database_name: Database name
    :param table_name: database table name
    :return: database information
    """
    return _get_info(dao_map, "GetModel", {"data_base_name": database_name, "table_name": table_name})


def _get_info(dao_map: DAOMap, query_name: str, query_param: dict) -> DataBaseInfo:
    """
    Get database information
    :param dao_map: the database info dao
    :param query_name: table manager query name
    :param query_param: table manager query name && query param
    :return: database information
    """
    # builder manager
    table_manager = dao_map["orm_table_info_dao"]
    column_manager = dao_map["orm_column_info_dao"]
    index_manager = dao_map["orm_index_info_dao"]
    key_manager = dao_map["orm_key_info_dao"]

    # Query table structure information
    tables = table_manager.query(query_name, query_param)
    for table in tables:
        table["columns"] = column_manager.query(
            "GetList",
            {"data_base_name": query_param["data_base_name"], "table_name": table["Name"]}
        )
        table["indexs"] = index_manager.query(
            "GetList",
            {"data_base_name": query_param["data_base_name"], "table_name": table["Name"]}
        )
        table["keys"] = key_manager.query(
            "GetList",
            {"data_base_name": query_param["data_base_name"], "table_name": table["Name"]}
        )
    return {"Name": query_param["data_base_name"], "tables": tables}



import pymysql
from superorm.engine import ExecuteEngine, ConnHandle, QueryResult, CountResult, ExecResult, ConnBuilder
from superorm.logger import Logger


class MySQLConnBuilder(ConnBuilder):

    host = None
    port = None
    user = None
    password = None
    database = None
    charset = None

    def __init__(self, host: str, user: str, password: str, database: str, port=3306, charset="utf8"):
        """
        Init MySQL Conn Builder
        :param host: host
        :param user: user
        :param password: password
        :param database: database
        :param port: data source port
        :param charset: charset
        """
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.database = database
        self.charset = charset

    def connect(self) -> pymysql.Connection:
        """
        Gets the database connection method
        """
        return pymysql.connect(
            host=self.host,
            port=self.port,
            user=self.user, password=self.password,
            database=self.database,
            charset=self.charset)


class MySQLConnHandle(ConnHandle):

    def ping(self, conn: pymysql.Connection):
        """
        Test whether the connection is available, and reconnect
        :param conn: database conn
        """
        conn.ping(reconnect=True)

    def commit(self, conn: pymysql.Connection):
        """
        Commit the connection
        :param conn: database conn
        """
        conn.commit()

    def rollback(self, conn: pymysql.Connection):
        """
        Rollback the connection
        :param conn: database conn
        """
        conn.rollback()


class MySQLExecuteEngine(ExecuteEngine):
    """
    MySQL Execution Engine
    """
    def query(self, conn: pymysql.Connection, logger: Logger, sql: str, parameter: list) -> QueryResult:
        """
        Query list information
        :param conn database conn
        :param logger logger
        :param sql: SQL statement to be executed
        :param parameter: parameter
        :return: Query results
        """
        cursor = conn.cursor(cursor=pymysql.cursors.DictCursor)

        exception = None
        try:
            logger.print_info(sql, parameter)
            cursor.execute(sql, parameter)
        except Exception as e:
            logger.print_error(e)
            cursor.close()
            raise exception

        # get result
        result = cursor.fetchall()
        # Close cursor
        cursor.close()
        return result

    def count(self, conn: pymysql.Connection, logger: Logger, sql: str, parameter: list) -> CountResult:
        """
        Query quantity information
        :param conn database conn
        :param logger logger
        :param sql: SQL statement to be executed
        :param parameter: parameter
        :return: Query results
        """
        result = self.query(conn, logger, sql, parameter)
        if len(result) == 0:
            return 0
        for value in result[0].values():
            return value

    # noinspection SpellCheckingInspection
    def exec(self, conn: pymysql.Connection, logger: Logger, sql: str, parameter: list) -> ExecResult:
        """
        Execute SQL statement
        :param conn database conn
        :param logger logger
        :param sql: SQL statement to be executed
        :param parameter: parameter
        :return: Last inserted ID, affecting number of rows
        """
        cursor = conn.cursor()

        exception = None
        try:
            logger.print_info(sql, parameter)
            cursor.execute(sql, parameter)
        except Exception as e:
            logger.print_error(e)
            cursor.close()
            raise exception

        # Number of rows affected
        rowcount = cursor.rowcount
        # Last insert ID
        lastrowid = cursor.lastrowid
        # Close cursor
        cursor.close()
        return lastrowid, rowcount



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()
    # noinspection SpellCheckingInspection
    setup(
        name="mysqlconn",
        version="1.0.2",
        description=(
            "MySQL query tool class used by python."
        ),
        keywords="mysql orm",
        long_description=long_description,
        long_description_content_type="text/markdown",
        author='lyoshur',
        author_email='1421333878@qq.com',
        maintainer='lyoshur',
        maintainer_email='1421333878@qq.com',
        license='MIT License',
        packages=find_packages(),
        platforms=["ubuntu", 'windows'],
        url='https://github.com/lyoshur/myorm',
        classifiers=[
            'Development Status :: 4 - Beta',
            'Operating System :: OS Independent',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python',
            'Programming Language :: Python :: Implementation',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Topic :: Software Development :: Libraries'
        ],
        install_requires=[
            'superorm==1.0.2',
            'PyMySQL',
        ],
        zip_safe=False
    )
